#!/usr/bin/perl -w

###########################################################################
# submit_translation.pl
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

#SYNOPSIS OF CODE:
#Processes translation when user clicks the 'SUBMIT TRANSLATION >>' button on
#greenstone site. Takes the cgiargsclass and the language being translated
#into, extracts the macronames and macrotext from the cgi data and hashes
#them, then links this hash to a database and updates database appropriately

BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib");
}

use parsargv;
use util;
use Cwd;
use File::Basename;
use GDBM_File;

sub main
{
    #gets the name of the args file with the cgiargsclass data in it
    my $filename = shift(@_);
    #gets the name of the language currently being translated from base language
    my $language = shift(@_);

    my $baselanguage = shift(@_);

    #stores a hash of macronames with numbers still appended and their values
    my %distinctkeys = ();
    #stores an hash of macronames without numbers appended and the value
    #being the full text of the macro
    my %macronames = ();
    
    my $macrotext = "";
    $number = 0;
    
    $langcode = &getlangcode($language);

    $language = "port" if ($language =~ m/portuguese/);
    $language = "indo" if ($language =~ m/indonesian/);
    
    my ($macronumbered) = &parsebasefile($baselanguage);
    my ($macronumbered2) = &parsebasefile($baselanguage . "2");
    ($macronumbered) = &combine_hashes($macronumbered, $macronumbered2);
 
    my ($foreign) = &parseforeignfile($language);
    my ($foreign2) = &parseforeignfile($language . "2");
    ($foreign) = &combine_hashes($foreign, $foreign2);

    my ($macronames) = &parseusersubmission($filename);

    foreach $key (keys %macronames) {
	if ($key =~ m/(_\w+_)/) {
	    $macronames{$key} = " [l=$langcode] {" . $macronames{$key} . "}\n";
	    $macro = $key;
	    $macro =~ s/.*:://;
	    if ($macronumbered{$macro}) {
		$number = $macronumbered{$macro};
		$foreign{$number} = $macro . $macronames{$key};
	    }
	}
	else {
	    $macronames{$key} =~ s/\&lt;br\&gt;/\n/g;
	    $macro = $key;
	    $macro =~ s/.*:://;
	    if ($macronumbered{$macro}) {
		$number = $macronumbered{$macro};
		$foreign{$number} = $macronames{$key};
	    }
	}
    }

    $language2 = $language . "2";

    open MACROOUT, ">$ENV{'GSDLHOME'}/macros/$language.dm" or die "MURGH\n";

    foreach $ky (sort {$a <=> $b} keys %foreign) {
	if ($foreign{$ky} =~ m/package\s*home/) {
	    close MACROOUT;
	    open MACROOUT, ">$ENV{'GSDLHOME'}/macros/$language2.dm" or die "MURGH\n";
	}
	print MACROOUT "$foreign{$ky}\n";
	
    }

    close MACROOUT;

    return 0;

}

&main(@ARGV);

sub getlangcode {

    $language = shift (@_);

    open(LANGFILE, "<$ENV{'GSDLHOME'}/tmp/lang/package_forms/languages.log") or die "MURGH\n";

    while (<LANGFILE>) {
	$lang = $_;
	chomp($lang);
	$langcode = <LANGFILE>;
	chomp($langcode);
	$langHASH{$lang} = $langcode;
    }
    
    close LANGFILE;
 
    $langcode = $langHASH{$language};
    
    return ($langcode);
}

sub parsebasefile {

    $file = shift(@_);

    open (BASEIN, "<$ENV{'GSDLHOME'}/macros/$file.dm") or die("MURGH\n");
    
    while (<BASEIN>) {
	
	$line = $_;
	chomp($line);
	$number++;

	if ($line =~ m/\Apackage /) {
	    $currpackage = $line;
	    $currpackage =~ s/package //;
	    $key = $currpackage . "::package";
	    $macronumbered{$key} = $number;
	}
	elsif ($line =~ m/\A\#\# .*/) {
	    
	    my @icon = split(/\s*\#\#\s*/, $line);
	    
	    $macro = $icon[(scalar @icon) - 1];
	    
	    $macronumbered{$macro} = $number;
	    
	    while ($line =~ m/\s+/) {
		$line = <BASEIN>;
		chomp($line);
	    }
	}
	elsif ($line =~ m/\A(_\w+_)/) {
	    $line = $&;
	    $macronumbered{$line} = $number;
	}
	elsif ($line =~ m/\A(\#\stext\smacros)/) {
	    $key = $currpackage . "::textmacro";
	    $macronumbered{$key} = $number;
	}
	elsif ($line =~ m/\A(\#\sicons)/) {
	    $key = $currpackage . "::icons";
	    $macronumbered{$key} = $number;
	}
    }
   
    close BASEIN;
    
    return (\%macronumbered);

}

sub parseforeignfile {

    $file = shift(@_);

    open (MACROIN, "<$ENV{'GSDLHOME'}/macros/$file.dm") or die("MURGH\n");
    
    
    while (<MACROIN>) {

	$number = 0;

	if ($_ =~ m/\A(_\w+_)/) {
   
	    $macro = $&;
	    $number = $macronumbered{$macro} if($macronumbered{$macro});
	    $line = $_;

	    # while there is still text of the macro to go...
	    while ($_ !~ /.*\}/) {
		# ... adds it to the macrotext variable
		$_ = <MACROIN>;
		$line .= $_;
	    }
	    if ($number != 0) {
		$foreign{$number} = $line;
	    }
	}
	elsif ($_ =~ m/\A\#\# .*/) {
	    $line = $_;
	    
	    $line .= <MACROIN> unless ($line =~ m/^\#\# .*\#\#/);

	    my @icon = split(/\s*\#\#\s*/, $line);
	    
	    $macro = $icon[(scalar @icon) - 1];
	    
	    $number = $macronumbered{$macro} if($macronumbered{$macro});
	    $currline = $line;
	    
	    while ($currline =~ m/\S+/) {
		$currline = <MACROIN>;
		$line .= $currline;
	    }
	    if ($number != 0) {
		$foreign{$number} = $line;
	    }
	}
	else {
	    if ($_ =~ m/\S+/) {
		$line = $_;
		if ($line =~ m/\Apackage /) {
		    $currpackage = $line;
		    chomp($currpackage);
		    $currpackage =~ s/package //;
		    $key = $currpackage . "::package";
		    $number = $macronumbered{$key} if($macronumbered{$key});
		    $line = "###################################################################\n" . $line;
		    $line .= "###################################################################\n";
		    $foreign{$number} = $line;
		}
		elsif ($line =~ m/\A(\#\stext\smacros)/) {
		    $key = $currpackage . "::textmacro";
		    $number = $macronumbered{$key} if($macronumbered{$key});
		    $line = "#-------------------------------------------------------------------\n" . $line;
		    $line .= "#-------------------------------------------------------------------\n";
		    $foreign{$number} = $line;
		}
		elsif ($line =~ m/\A(\#\sicons)/) {
		    $key = $currpackage . "::icons";
		    $number = $macronumbered{$key} if($macronumbered{$key});
		    $line = "#-------------------------------------------------------------------\n" . $line;
		    $line .= "#-------------------------------------------------------------------\n";
		    $foreign{$number} = $line;
		}
	    }
	}
    }
    close MACROIN;

    return (\%foreign);

}


sub parseusersubmission {

    $filename = shift(@_);
    
    #opens the args file or kills program
    open(ARGUMENTS, "<$filename") or die("Did not open $filename\n");
    #while there are still lines left to read
    while (<ARGUMENTS>) {
	#gets the line
	$line = $_;
	#chomps off the final newline character
	chomp($line);
	
        #if the line contains :: (browser converts to %3A%3A) then it has a macroname
	if ($line =~ m/%3A%3A/) {
	    $macro = $line;
	    #changes the codes back to ::
	    $macro =~ s/%3A%3A/::/;
	    #removes value of macro leaving just the name
	    $macro =~ s/=.*//;
	    #saves the value
	    $value = $&;
	    #removes all " and = from value
	    $value =~ s/=//g;
	    $value =~ s/\A\"//;
	    $value =~ s/\"\Z//;
	    $value =~ s/<br>/\n/g;
	    #removes all " from macro
	    $macro =~ s/\"//g;
	    #removes any whitespace from macro
	    $macro =~ s/\s+//g;
	    #hashes the macro and it's value
	    if ($value =~ m/\S+/) {
		$distinctkeys{$macro} = $value;
	    }
	}
    }

    foreach $distinctkey (sort keys %distinctkeys) {

	$key = $distinctkey;
	$key =~ s/\d+//;
	$number = $&;
	
	$orderingHASH{$key}{$number} = $distinctkeys{$distinctkey};
    }

    for $key (sort keys %orderingHASH) {
	
	for $number (sort {$a <=> $b} keys %{$orderingHASH{$key}}) {
	    $macrotext .= $orderingHASH{$key}{$number};
	    
	}
	
	$macronames{$key} = $macrotext;
	$macrotext = "";
    }

    close ARGUMENTS;

    return (\%macronames);
}

sub combine_hashes {
    
    my ($hash1, $hash2) = @_;
    my %combined = ();

    foreach $key (keys %$hash1) {
	$combined{$key} = $hash1->{$key};
    }
    foreach $key (keys %$hash2) {
	$combined{$key} = $hash2->{$key};
    }

    return (\%combined);

}



