package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.io.File;

/** Action class for retrieving Documents  via the message router
 */
public class DocumentAction extends Action {
  
   public Element process (Element message) {
	
	// find the stylesheet
	String stylesheet = GSFile.stylesheetFile(config_, "document.xsl");

	if (stylesheet==null) {
	    System.err.println("DocumentAction Error: document stylesheet not found!");
	    return null;
	}
	
	// for now, no subaction eventually we may want to have subactions such as text assoc or something ?
	
	// get the request - assume only one
	Element request = (Element)GSXML.getChildByTagName(message, GSXML.REQUEST_ELEM);
	Element cgi_paramList = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_paramList);

	String document_name = (String)params.get(GSCGI.DOCUMENT_ARG);
	if (document_name == null || document_name.equals("")) {
	    System.err.println("DocumentAction Error: no document specified!");
	    return null;
	}
		
	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the config stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));

	// build up the mr request
	Element mr_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_message.appendChild(mr_request);
	mr_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_QUERY);
	mr_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	String to = (String)params.get(GSCGI.COLLECTION_ARG); // collection name
	to = GSPath.appendLink(to, "DocumentRetrieve");
	mr_request.setAttribute(GSXML.TO_ATT, to);

	// all query requests must have a paramList and a content
	Element paramList = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	mr_request.appendChild(paramList);

	Element content = doc_.createElement(GSXML.CONTENT_ELEM);
	mr_request.appendChild(content);

	Element document_list = doc_.createElement(GSXML.DOCUMENT_ELEM+GSXML.LIST_MODIFIER);
	Element document = doc_.createElement(GSXML.DOCUMENT_ELEM);
	document.setAttribute(GSXML.NAME_ATT, document_name);
	document_list.appendChild(document);
	content.appendChild(document_list);

	Element mr_result = (Element)mr_.process(mr_message);

	// add the results to the page
	page.appendChild(doc_.importNode(request, true));
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(mr_result, GSXML.RESPONSE_ELEM), true));
	
	// process using the stylesheet
	Document style_doc = converter_.getDOM(new File(stylesheet));
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    

    }
    

}
