package org.greenstone.gsdl3.util;

import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Text;

import java.util.HashMap;
//import java.util.Locale;

/** various functions for extracting info out of GS XML */
public class GSXML {

    // greenstone xml elements
    public static final String MESSAGE_ELEM = "message";
    public static final String REQUEST_ELEM = "request";
    public static final String RESPONSE_ELEM = "response";
    public static final String COLLECTION_ELEM = "collection";
    public static final String SERVICE_ELEM = "service";
    public static final String CLUSTER_ELEM = "serviceCluster";
    public static final String SITE_ELEM = "site";
    public static final String PARAM_ELEM = "param";
    public static final String PARAM_OPTION_ELEM = "option";
    public static final String CONTENT_ELEM = "content";
    public static final String RESOURCE_ELEM = "resource";
    public static final String DOCUMENT_ELEM = "document";
    public static final String METADATA_ELEM = "metadata";
    public static final String SERVICE_CLASS_ELEM = "serviceRack";
    public static final String CLASSIFIER_ELEM = "classifier";
    public static final String APPLET_ELEM = "applet";
    public static final String APPLET_DATA_ELEM = "appletData";
    public static final String CONFIGURE_ELEM = "configure";
    public static final String STATUS_ELEM = "status";
    public static final String ERROR_ELEM = "error";
    public static final String DEFAULT_ELEM = "default";
    public static final String STYLESHEET_ELEM = "stylesheet"; // any additional stylesheet stuff is carried in the message inside this elem

    // elems for the pages to be processed by xslt
    public final static String PAGE_ELEM = "page";
    public final static String TRANSLATION_ELEM = "translate";
    public final static String CONFIGURATION_ELEM = "config";
    public final static String DESCRIPTION_ELEM = "description";

    public static final String SITE_NAME_ELEM = "localSiteName";

    // add on to another elem type to get a list of that type
    public static final String LIST_MODIFIER = "List";

    // greenstone xml attributes
    public static final String NAME_ATT = "name";
    public static final String TO_ATT = "to";
    public static final String FROM_ATT = "from";
    public static final String LANG_ATT = "lang";
    public static final String TYPE_ATT = "type";
    public static final String VALUE_ATT = "value";
    public static final String DEFAULT_ATT = "default";
    public static final String INFO_ATT = "info";
    public static final String ACTION_ATT = "action";
    public static final String SUBACTION_ATT = "subaction";
    public static final String OUTPUT_ATT = "output";
    public static final String ADDRESS_ATT = "address";
    public static final String STATUS_ERROR_CODE_ATT = "code";
    public static final String STATUS_PROCESS_ID_ATT = "handle";
    public static final String PARAM_SHORTNAME_ATT = "shortname";
    public static final String PARAM_IGNORE_POS_ATT = "ignore";

    // parameter types
    public static final String PARAM_TYPE_INTEGER = "integer";
    public static final String PARAM_TYPE_BOOLEAN = "boolean";
    public static final String PARAM_TYPE_ENUM = "enum";
    public static final String PARAM_TYPE_ENUM_SINGLE = "enum_single";
    public static final String PARAM_TYPE_ENUM_MULTI = "enum_multi";
    public static final String PARAM_TYPE_STRING = "string";
    public static final String PARAM_TYPE_TEXT = "text";
    public static final String PARAM_TYPE_MULTI = "multi";
    public static final String PARAM_TYPE_FILE = "file";
    // stuff for text strings 
    public static final String DISPLAY_ELEM = "display";
    public static final String DISPLAY_NAME_ELEM = "name";
    public static final String DISPLAY_SUBMIT_ELEM = "submit";

    // request types - do we need all these? or just
    // describe, status, and one to say do teh request
    public static final String REQUEST_TYPE_DESCRIBE = "describe";
    public static final String REQUEST_TYPE_CONFIGURE = "configure";
    public static final String REQUEST_TYPE_QUERY = "query";
    public static final String REQUEST_TYPE_ACTION = "action";
    public static final String REQUEST_TYPE_BUILD = "build";
    public static final String REQUEST_TYPE_STATUS = "status";
    public static final String REQUEST_TYPE_PROCESS = "process";
    public static final String REQUEST_TYPE_CGI = "cgi";
   
    // service types
    public static final String SERVICE_TYPE_QUERY = "query";
    public static final String SERVICE_TYPE_RETRIEVE = "retrieve";
    public static final String SERVICE_TYPE_BROWSE = "browse";
    public static final String SERVICE_TYPE_APPLET = "applet";
    public static final String SERVICE_TYPE_PROCESS = "process";

    // configure types
    public static final String CONFIG_ACTION_ACTIVATE = "activate";
    public static final String CONFIG_ACTION_DEACTIVATE = "deactivate";
   
    // communicator types
    public static final String COMM_TYPE_SOAP_JAVA = "soap";
    
    // takes a node with a resource elements inside it and extracts all the 
    // HASh oids - name att for resource
    public static String [] getDocumentNameList(Element content) {
	
	Node n = content.getFirstChild();
	while (n!=null && !n.getNodeName().equals(DOCUMENT_ELEM+LIST_MODIFIER)) {
	    n = n.getNextSibling();
	}
	if (n==null) { // no docs found
	    return null;
	}
	    
	NodeList docs = n.getChildNodes();
	
	int numdocs = docs.getLength();
	String []ids = new String[numdocs];
	for (int i=0; i<numdocs; i++) {
	    Element e = (Element)docs.item(i);
	    String id = e.getAttribute(NAME_ATT);
	    // check that its a valid id - ie starts with HASH 
	    // need to change this if use different ids
	    
	    ids[i] = id;
	    
	}

	return ids;
    }

    /** extracts metadata names out of an element */
    public static String [] getMetaNameList(Element content) {
	Node n = content.getFirstChild();
	while (n!=null && 
	       !n.getNodeName().equals(METADATA_ELEM+LIST_MODIFIER)) {
	    n = n.getNextSibling();
	}
	if (n==null) { // no metadatas found
	    return null;
	}
	NodeList elems = n.getChildNodes();
	
	int numelems = elems.getLength();
	String []ids = new String[numelems];
	for (int i=0; i<numelems; i++) {
	    Element e = (Element)elems.item(i);
	    String id = e.getAttribute(NAME_ATT);
	    ids[i] = id;
	}

	return ids;
    }

    /** takes a paramList element, and gets a HashMap of name-value pairs */
    public static HashMap extractParams(Element xml) {

	if (!xml.getNodeName().equals(PARAM_ELEM+LIST_MODIFIER)) {
	    System.err.println("GSXML:paramList element should have been passed to extractParams, instead it was "+xml.getNodeName());
	    return null;
	}
	NodeList params = xml.getChildNodes();
	HashMap param_map = new HashMap();
	for (int i=0; i<params.getLength(); i++) {
	    Element param = (Element)params.item(i);
	    String name=param.getAttribute(NAME_ATT);
	    String value=param.getAttribute(VALUE_ATT);
	    if (value.equals("")) { // the value is in the content of the param
		value=getNodeText(param); 
	    }
	    param_map.put(name, value);
	    
	}
	return param_map;
    }
    /** takes a paramList element, and gets a HashMap of name-value pairs */
    public static HashMap extractAllParams(Element xml) {

	if (!xml.getNodeName().equals(PARAM_ELEM+LIST_MODIFIER)) {
	    System.err.println("GSXML:paramList element should have been passed to extractParams, instead it was "+xml.getNodeName());
	    return null;
	}
	NodeList params = xml.getElementsByTagName(PARAM_ELEM);
	HashMap param_map = new HashMap();
	for (int i=0; i<params.getLength(); i++) {
	    Element param = (Element)params.item(i);
	    String name=param.getAttribute(NAME_ATT);
	    String value=param.getAttribute(VALUE_ATT);
	    if (value.equals("")) { // the value is in the content of the param
		value=getNodeText(param); 
	    }
	    param_map.put(name, value);
	    
	}
	return param_map;
    }

    public static String getValue(Element e) {
	String val = e.getAttribute(VALUE_ATT);
	if (val ==null || val.equals("")) {
	    // have to get it out of the text
	    val=getNodeText(e); 

	}
	return val;
    }
    /** extracts the text out of a node */
    public static String getNodeText(Element param) {
	param.normalize();
	Node n = param.getFirstChild();
	while (n!=null && n.getNodeType() !=Node.TEXT_NODE) {
	    n=n.getNextSibling();
	}
	if (n==null) { // no text node
	    return "";
	}
	return n.getNodeValue();
    }
    /** creates a new document Element */
    public static Element createDocumentElement(Document owner, String oid) { 
	Element e = owner.createElement(DOCUMENT_ELEM);
	e.setAttribute(NAME_ATT, oid);
	
	return e;
    }
	
    /** add text to a document/subsection  element */
    public static boolean addDocText(Document owner, Element doc, String text) {

	Element content = owner.createElement(CONTENT_ELEM);
	Text t = owner.createTextNode(text);
	content.appendChild(t);
	doc.appendChild(content);
	return true;
    }
  
    /** add an error message */
    public static boolean addError(Document owner, Element doc, String text) {

	Element content = owner.createElement(ERROR_ELEM);
	Text t = owner.createTextNode(text);
	content.appendChild(t);

	return true;
    }

    /** add an error message */
    public static boolean addError(Document owner, Element doc, Throwable error) {
      error.printStackTrace();
      return addError(owner, doc, error.toString());
    }

    /** adds an empty MetadataList elem to a doc, and returns a ref to it*/
    public static Element addMetaList(Document owner, Element doc) {
	Element list = owner.createElement(METADATA_ELEM+LIST_MODIFIER);
	doc.appendChild(list);
	return list;
    }
    /** adds a metadata elem to a list */
    public static boolean addMetadata(Document owner, Element list, 
				      String meta_name, String meta_value) {
	if (meta_value==null || meta_value.equals("")) {
	    return false;
	}
	Element data = owner.createElement(METADATA_ELEM);
	data.setAttribute(NAME_ATT, meta_name);
	Text t = owner.createTextNode(meta_value);
	data.appendChild(t);
	list.appendChild(data);
	return true;

    }

    public static boolean mergeMetadataLists(Node to, Node from) {
	Node to_meta = getChildByTagName(to, METADATA_ELEM+LIST_MODIFIER);
	Node from_meta = getChildByTagName(from, METADATA_ELEM+LIST_MODIFIER);
	
	if  (from_meta == null) { // nothing to copy
	    return true;
	}
	Document to_owner = to.getOwnerDocument();
	Node new_from = to_owner.importNode(from_meta, true);

	if (to_meta == null) { // just copy the whole list
	    to.appendChild(new_from);
	    return true;
	}
	
	// copy individual elements
	Node child = new_from.getFirstChild();
	while ( child != null) {
	    to_meta.appendChild(child);
	    child = child.getNextSibling();
	}
	return true;
    }
	
    /** takes two nodes - should have the same node type, tag name, etc
     * copies the info from 'from' into 'to' 
     -- not finished haven't thought through the algo yet.*/
    /*
    public static boolean mergeElements(Element to, Element from) {

	if (to.getNodeName()!=from.getNodeName()) {
	    System.err.println("cant merge two nodes with different names");
	    return false;
	}

	// copy any atts over - ignore for now

	// copy the children
	if (!from.hasChildNodes()) {
	    return true;
	}

	// check if they belong to the same document
	Document to_doc = to.getOwnerDocument();
	Document from_doc = from.getOwnerDocument();
	Element newfrom;
	if (to_doc != from_doc) {
	    nfrom = to_doc.importNode(from, true);
	} else {
	    newfrom = from;
	}
	
	if (!to.hasChildNodes()) {
	    // just copy all the children over
	    Node child = newfrom.getFirstChild();
	    while (child !=null) {
		to.appendChild(child);
		child = child.getNextSibling();
	    }
	    return true;
	}
	
	// have to check each one to see if already present or not
	HashMap children = getChildrenMap(to);
	
	Node child = newfrom.getFirstChild();
	while (child !=null) {
	    String name = child.getNodeName();
	    Node n = map.get(name);
	    if (n==null) { // there is no elem by that name in to
		to.appendChild(child);
	    }
	    else {
	}
	return true;
    }
    */
    /** returns the (first) child element with the given name */
    public static Node getChildByTagName(Node n, String name) {

	Node child = n.getFirstChild();
	while (child!=null) {
	    if (child.getNodeName().equals(name)) {
		return child;
	    }
	    child = child.getNextSibling();
	}
	return null; //not found
    }

    /** takes an xpath type expression of the form name/name/... 
	and returns the first node that matches, or null if not found */
    public static Node getNodeByPath(Node n, String path) {
	
	String link = GSPath.getFirstLink(path);
	path = GSPath.removeFirstLink(path);
	while (!link.equals("")) {
	    n = getChildByTagName(n, link);
	    if (n==null) {
		return null;
	    }
	    link = GSPath.getFirstLink(path);
	    path = GSPath.removeFirstLink(path);
	}
	return n;
    }
    public static HashMap getChildrenMap(Node n) {
	
	HashMap map= new HashMap();
	Node child = n.getFirstChild();
	while (child!=null) {
	    String name = child.getNodeName();
	    map.put(name, child);
	    child = child.getNextSibling();
	}
	return map;
    }
       
    public static Element createTextElement(Document owner, String elem_name,
					    String text) {
	Element e = owner.createElement(elem_name);
	Text t = owner.createTextNode(text);
	e.appendChild(t);
	return e;

    }


    public static Element createParameter(Document owner, String name,
					  String type, String default_value,
					  String []options) {


	Element p = owner.createElement(PARAM_ELEM);
	p.setAttribute(NAME_ATT, name);
	p.setAttribute(TYPE_ATT, type);
	if (default_value != null) {
	    p.setAttribute(DEFAULT_ATT, default_value);
	}
	if (type.startsWith(PARAM_TYPE_ENUM) && options!=null) {
	    for (int i=0; i<options.length; i++) {
		Element e = owner.createElement(PARAM_OPTION_ELEM);
		e.setAttribute(NAME_ATT, options[i]);
		p.appendChild(e);
	    }
	}
	return p;
    }

    /** creates a parm elem containing the text strings
     * <param><name>xxx<name><option name='x'>yyy</option><option name='y'>zzz</option></param>
     */
    public static Element createParameterDisplay(Document owner, String name,
						 String name_text, 
						 String []options,
						 String []option_texts) {
					     

	Element param = owner.createElement(PARAM_ELEM);
	param.setAttribute(NAME_ATT, name);
	param.appendChild(createTextElement(owner, DISPLAY_NAME_ELEM, name_text));
	if (options != null) {
	    for (int i=0; i<options.length; i++) {
		Element e = GSXML.createTextElement(owner, PARAM_OPTION_ELEM, option_texts[i]);
		e.setAttribute(NAME_ATT, options[i]);
		param.appendChild(e);
	    }
	}

    return param;
    }

    /** returns the element parent/node_name[@attribute_name='attribute_value']
     */
    public static Element getNamedElement(Element parent, String node_name,
					  String attribute_name, 
					  String attribute_value) {

	NodeList children = parent.getChildNodes();
	for (int i=0; i<children.getLength(); i++) {
	    Node child = children.item(i);
	    if (child.getNodeName().equals(node_name)) {
		if (((Element)child).getAttribute(attribute_name).equals(attribute_value))
		    return (Element)child;
	    }
	}
	// not found
	return null;
    }

    // replaces < > " ' & in the original with their entities
    public static String xmlSafe(String original) {

	StringBuffer filtered = new StringBuffer(original.length());
	char c;
	for (int i=0; i<original.length(); i++) {
	    c = original.charAt(i);
	    if (c == '>') {
		filtered.append("&gt;");
	    } else if (c == '<') {
		filtered.append("&lt;");
	    } else if (c == '"') {
		filtered.append("&quot;");
	    } else if (c == '&') {
		filtered.append("&amp;");
	    } else if (c == '\'') {
		filtered.append("&apos;");
	    } else {
		filtered.append(c);
	    }
	}
	return filtered.toString();
    }
}
